function [mae,rmse,re]=rederr(ybarc,ybarv,yhat,y)
% rederr:  verification statistics for a climate reconstruction
% [mae,rmse,re]=rederr(ybarc,ybarv,yhat,y);
% Last revised 10-15-03
%
%*** INPUT ARGS
%
% ybarc (1 x 1r  calibration period mean of observed predictand
% ybarv (1 x 1)r  verification-period mean of observed predictand
% yhat (m1 x 1)r  verification-period time series of reconstructed predictand
% y (m2 x 1)r verification-period time series of observed predictand
%
%*** OUTPUT ARGS
%
% mae (1 x 3) mean absolute error assuming the reconstruction is:
%      mae(1):  the reconstructed time series itself (no assuming here)
%      mae(2):  ybarc: the calibration-period mean of the observed data
%      mae(3):  ybarv: the verification-period mean of the observed data
% rmse (1 x 3) root-mean-square error, ordered as in mae
% re   (4 x 2) reduction-of-error statistic and its components 
%     re(1,:) --  based on mean square errors for the reconstruction and
%			compared with mse if reconstruction were a constant equal
%			to ybarc
%     re(2,:) --  based on mean square errors for the reconstruction and
%			compared with mse if reconstruction were a constant equal
%			to ybarv
%     row 1: RE (or CE)
%     row 2: risk
%     row 3: bias
%     row 4: covariance
%
%
%*** REFERENCES
% 
% The mae and rmse error are defined in many standard time series and regression texts, 
% for example:
% Weisberg, S., 1985, Applied Linear Regression, 2nd ed., John Wiley, New York.
%
% The re statistic in a tree-ring context is discussed in:
% Gordon, G., 1982, Verification of dendroclimatic reconstructions, In:  
% M.K. Hughes et al. (eds.), Climate from tree rings.  Cambridge University Press,
% Cambridge, UK, 58-61.
%
% The risk, bias and covariance components of RE are described in:
% Fritts, H.C., Guiot, J. and Gordon, G.A. (1990), Verification, in:
% Methods of dendrochronology; applications in the environmental sciences,
% eds E.R. Cook and L.A. Kairiukstis, Kluwer Academic Publshers, Boston
%
%
% Revision 10-15-03:  build in risk, bias and covariance components
%
%
%*** UW FUNCTIONS CALLED
%*** TOOLBOXES NEEDED
%
%*** NOTES
%
% Equation 4.40a, p. 182 in Fritts et al. (1990) has a typo.  The last
% term, for the covariance component, should be divided by the same sum of
% squares of yi as the first two terms.  
% 

%*****     Size arrays and do a few consistency checks

if nargin~=4  &  nargout~=3
	error('Incorrect number of input or output arguments')
end

[m1,n1]=size(yhat);
[m2,n2]=size(y);
L1 = [n1~=1  n2~=1 m1~=m2];
if any(L1)
	error('yhat, y must be same-length CVs')
end

n=length(y);  % sample size


% Compute errors of reconstruction, mean abs errors,mean square err
% and root-mean-square error

e1=y-yhat;   % reconstruction error
e2=y-ybarc;  % errors if ybarc were reconstruction
e3=y-ybarv;  % errors if ybarv were reconstruction

mae=[mean(abs(e1))  mean(abs(e2))  mean(abs(e3))];
mserr=[e1'*e1   e2'*e2  e3'*e3] / n;  % mean square errors
rmse = sqrt(mserr);

% Compute re statistics

re=repmat(NaN,4,2);

re(1,1) = (mserr(2) - mserr(1))/ mserr(2);
re(1,2) = (mserr(3) - mserr(1))/ mserr(3);



% Risk, bias and covariance components of RE, with ybarc as the comparison
% null prediction
% Begin with predicted and observed values expressed as departures
% calibration-period mean
z=y-ybarc;
zhat=yhat-ybarc;
REfritts = 1.0-   ((z-zhat)' * (z-zhat))/  (z' * z); % Compute RE by eq. 4.39, Fritts et al. 1990


RE_risk=-1.0 * (zhat' * zhat) / (z' * z); % Risk

RE_bias = (2 * n * mean(z) * mean(zhat)) / (z' *z); % bias

g=cov(z,zhat);
g=g(1,2);
RE_cov = (2 * (n-1) *  g)/(z' * z); % covariance
doff = abs(REfritts-(RE_risk+RE_bias+RE_cov));
if doff>1E-6;
    error('Risk, bias, cov do not sum to RE in ybarc version');
end;
re([2 3 4],1)=[RE_risk RE_bias RE_cov]';



% Risk, bias and covariance components of RE, with ybarv as the comparison
% null prediction
% Begin with predicted and observed values expressed as departures
% calibration-period mean
z=y-ybarv;
zhat=yhat-ybarv;
REfritts = 1.0-   ((z-zhat)' * (z-zhat))/  (z' * z); % Compute RE by eq. 4.39, Fritts et al. 1990


RE_risk=-1.0 * (zhat' * zhat) / (z' * z); % Risk

RE_bias = (2 * n * mean(z) * mean(zhat)) / (z' *z); % bias

g=cov(z,zhat);
g=g(1,2);
RE_cov = (2 * (n-1) *  g)/(z' * z); % covariance
doff = abs(REfritts-(RE_risk+RE_bias+RE_cov));
if doff>1E-6;
    error('Risk, bias, cov do not sum to RE in ybarv version');
end;
re([2 3 4],2)=[RE_risk RE_bias RE_cov]';
